@@ -126,6 +126,11 @@ validate (GisPasswordPage *page)
   gtk_level_bar_set_value (GTK_LEVEL_BAR (priv->password_strength), strength_level);
   gtk_label_set_label (GTK_LABEL (priv->password_explanation), long_hint);
 
+  if (strlen (password) > 0 && strength_level <= 0)
+    set_entry_validation_error (GTK_ENTRY (priv->password_entry), _("This is a weak password."));
+  else
+    clear_entry_validation_error (GTK_ENTRY (priv->password_entry));
+
   gtk_label_set_label (GTK_LABEL (priv->confirm_explanation), "");
   priv->valid_confirm = FALSE;
 
@@ -243,6 +248,7 @@ gis_password_page_constructed (GObject *object)
 static void
 gis_password_page_dispose (GObject *object)
 {
+  if (GIS_PAGE (object)->driver)
   g_signal_handlers_disconnect_by_func (GIS_PAGE (object)->driver,
                                         username_changed, object);
 
